import unittest
from datetime import datetime

class Thermostat:

	def __init__(self):
		self.setPoints = []

	def GetCurrentRequiredTemperature(self):
		time = int(datetime.now().strftime("%H"))
		return self.GetCurrentRequiredTemperatureAtTime(time)

	def GetCurrentRequiredTemperatureAtTime(self, time):
		passedSetPoints = [sp for sp in self.setPoints if sp[0] <= time]
		lastPassedSetPoint = max(passedSetPoints, default = (0, 10), key = lambda sp: sp[0])
		return lastPassedSetPoint[1]

	def AddSetPoint(self, hour, temperature):
		self.setPoints.append((hour, temperature))

class ThermostatTests(unittest.TestCase):

	def test_GivenNewThermostat_ThenRequiredTemperatureIs10(self):
		thermostat = Thermostat()
		currentRequiredTemperature = thermostat.GetCurrentRequiredTemperature()
		self.assertEqual(currentRequiredTemperature, 10)

	def test_GivenIAddedASetPoint_WhenTimeIsAfterSetPoint_ThenRequiredTemperatureIsChanged(self):
		hour = 10
		requiredTemperature = 30
		thermostat = Thermostat()
		thermostat.AddSetPoint(hour, requiredTemperature)
		self.assertEqual(thermostat.GetCurrentRequiredTemperatureAtTime(hour), requiredTemperature)

	def test_GivenIAddedSetPoint_WhenTimeIsBeforeSetPoint_ThenRequiredTemperatureNotChanged(self):
		hour = 7
		requiredTemperature = 25
		thermostat = Thermostat()
		oldRequiredTemperature = thermostat.GetCurrentRequiredTemperatureAtTime(hour-1)
		thermostat.AddSetPoint(hour, requiredTemperature)
		newRequiredTemperature = thermostat.GetCurrentRequiredTemperatureAtTime(hour-1)
		self.assertEqual(oldRequiredTemperature, newRequiredTemperature)

	def test_GivenTwoSetPoints_WhenTimeIsBetweenThem_ThenRequiredTemperatureIsFirst(self):
		hour1, hour2 = 12, 15
		temp1, temp2 = 55, 75
		thermostat = Thermostat()
		thermostat.AddSetPoint(hour1, temp1)
		thermostat.AddSetPoint(hour2, temp2)
		self.assertEqual(thermostat.GetCurrentRequiredTemperatureAtTime(14), 55)
