import unittest
from datetime import datetime

class Thermostat:

	def __init__(self):
		self.setPoints = []

	def GetCurrentTime(self):
		return int(datetime.now().strftime("%H"))

	def GetCurrentRequiredTemperature(self):
		time = self.GetCurrentTime()
		passedSetPoints = [sp for sp in self.setPoints if sp[0] <= time]
		lastPassedSetPoint = max(passedSetPoints, default = (0, 10), key = lambda sp: sp[0])
		return lastPassedSetPoint[1]

	def AddSetPoint(self, hour, temperature):
		self.setPoints.append((hour, temperature))

class TestableThermostat(Thermostat):

	def __init__(self, hour):
		self.hour = hour
		self.setPoints = []

	def GetCurrentTime(self):
		return self.hour

class ThermostatTests(unittest.TestCase):

	def test_GivenNewThermostat_ThenRequiredTemperatureIs10(self):
		thermostat = Thermostat()
		currentRequiredTemperature = thermostat.GetCurrentRequiredTemperature()
		self.assertEqual(currentRequiredTemperature, 10)

	def test_GivenIAddedASetPoint_WhenTimeIsAfterSetPoint_ThenRequiredTemperatureIsChanged(self):
		hour = 10
		requiredTemperature = 30
		thermostat = TestableThermostat(hour)
		thermostat.AddSetPoint(hour, requiredTemperature)
		self.assertEqual(thermostat.GetCurrentRequiredTemperature(), requiredTemperature)

	def test_GivenIAddedSetPoint_WhenTimeIsBeforeSetPoint_ThenRequiredTemperatureNotChanged(self):
		hour = 7
		requiredTemperature = 25
		thermostat = TestableThermostat(hour-1)
		oldRequiredTemperature = thermostat.GetCurrentRequiredTemperature()
		thermostat.AddSetPoint(hour, requiredTemperature)
		newRequiredTemperature = thermostat.GetCurrentRequiredTemperature()
		self.assertEqual(oldRequiredTemperature, newRequiredTemperature)

	def test_GivenTwoSetPoints_WhenTimeIsBetweenThem_ThenRequiredTemperatureIsFirst(self):
		hour1, hour2 = 12, 15
		temp1, temp2 = 55, 75
		thermostat = TestableThermostat(14)
		thermostat.AddSetPoint(hour1, temp1)
		thermostat.AddSetPoint(hour2, temp2)
		self.assertEqual(thermostat.GetCurrentRequiredTemperature(), 55)
